@@ -1,11 +1,16 @@
 import itertools
 import time
+from typing import Dict, Optional, Union
 
-from .document import Document
-from .result import Result
-from .query import Query
+from redis.client import Pipeline
+from redis.utils import deprecated_function
+
+from ..helpers import parse_to_dict
 from ._util import to_string
 from .aggregation import AggregateRequest, AggregateResult, Cursor
+from .document import Document
+from .query import Query
+from .result import Result
 from .suggestion import SuggestionParser
 
 NUMERIC = "NUMERIC"
@@ -16,10 +21,12 @@ SEARCH_CMD = "FT.SEARCH"
 ADD_CMD = "FT.ADD"
 ADDHASH_CMD = "FT.ADDHASH"
 DROP_CMD = "FT.DROP"
+DROPINDEX_CMD = "FT.DROPINDEX"
 EXPLAIN_CMD = "FT.EXPLAIN"
 EXPLAINCLI_CMD = "FT.EXPLAINCLI"
 DEL_CMD = "FT.DEL"
 AGGREGATE_CMD = "FT.AGGREGATE"
+PROFILE_CMD = "FT.PROFILE"
 CURSOR_CMD = "FT.CURSOR"
 SPELLCHECK_CMD = "FT.SPELLCHECK"
 DICT_ADD_CMD = "FT.DICTADD"
@@ -42,7 +49,12 @@ SYNDUMP_CMD = "FT.SYNDUMP"
 
 NOOFFSETS = "NOOFFSETS"
 NOFIELDS = "NOFIELDS"
+NOHL = "NOHL"
+NOFREQS = "NOFREQS"
+MAXTEXTFIELDS = "MAXTEXTFIELDS"
+TEMPORARY = "TEMPORARY"
 STOPWORDS = "STOPWORDS"
+SKIPINITIALSCAN = "SKIPINITIALSCAN"
 WITHSCORES = "WITHSCORES"
 FUZZY = "FUZZY"
 WITHPAYLOADS = "WITHPAYLOADS"
@@ -64,6 +76,11 @@ class SearchCommands:
         no_field_flags=False,
         stopwords=None,
         definition=None,
+        max_text_fields=False,
+        temporary=None,
+        no_highlight=False,
+        no_term_frequencies=False,
+        skip_initial_scan=False,
     ):
         """
         Create the search index. The index must not already exist.
@@ -77,15 +94,39 @@ class SearchCommands:
         allow searching in specific fields
         - **stopwords**: If not None, we create the index with this custom
         stopword list. The list can be empty
-        """
+        - **max_text_fields**: If true, we will encode indexes as if there
+        were more than 32 text fields which allows you to add additional
+        fields (beyond 32).
+        - **temporary**: Create a lightweight temporary index which will
+        expire after the specified period of inactivity (in seconds). The
+        internal idle timer is reset whenever the index is searched or added to.
+        - **no_highlight**: If true, disabling highlighting support.
+        Also implied by no_term_offsets.
+        - **no_term_frequencies**: If true, we avoid saving the term frequencies
+        in the index.
+        - **skip_initial_scan**: If true, we do not scan and index.
+
+        For more information see `FT.CREATE <https://redis.io/commands/ft.create>`_.
+        """  # noqa
 
         args = [CREATE_CMD, self.index_name]
         if definition is not None:
             args += definition.args
+        if max_text_fields:
+            args.append(MAXTEXTFIELDS)
+        if temporary is not None and isinstance(temporary, int):
+            args.append(TEMPORARY)
+            args.append(temporary)
         if no_term_offsets:
             args.append(NOOFFSETS)
+        if no_highlight:
+            args.append(NOHL)
         if no_field_flags:
             args.append(NOFIELDS)
+        if no_term_frequencies:
+            args.append(NOFREQS)
+        if skip_initial_scan:
+            args.append(SKIPINITIALSCAN)
         if stopwords is not None and isinstance(stopwords, (list, tuple, set)):
             args += [STOPWORDS, len(stopwords)]
             if len(stopwords) > 0:
@@ -107,7 +148,9 @@ class SearchCommands:
         ### Parameters:
 
         - **fields**: a list of Field objects to add for the index
-        """
+
+        For more information see `FT.ALTER <https://redis.io/commands/ft.alter>`_.
+        """  # noqa
 
         args = [ALTER_CMD, self.index_name, "SCHEMA", "ADD"]
         try:
@@ -117,17 +160,6 @@ class SearchCommands:
 
         return self.execute_command(*args)
 
-    def drop_index(self, delete_documents=True):
-        """
-        Drop the index if it exists. Deprecated from RediSearch 2.0.
-
-        ### Parameters:
-
-        - **delete_documents**: If `True`, all documents will be deleted.
-        """
-        keep_str = "" if delete_documents else "KEEPDOCS"
-        return self.execute_command(DROP_CMD, self.index_name, keep_str)
-
     def dropindex(self, delete_documents=False):
         """
         Drop the index if it exists.
@@ -137,9 +169,11 @@ class SearchCommands:
         ### Parameters:
 
         - **delete_documents**: If `True`, all documents will be deleted.
-        """
-        keep_str = "" if delete_documents else "KEEPDOCS"
-        return self.execute_command(DROP_CMD, self.index_name, keep_str)
+
+        For more information see `FT.DROPINDEX <https://redis.io/commands/ft.dropindex>`_.
+        """  # noqa
+        delete_str = "DD" if delete_documents else ""
+        return self.execute_command(DROPINDEX_CMD, self.index_name, delete_str)
 
     def _add_document(
         self,
@@ -152,13 +186,11 @@ class SearchCommands:
         partial=False,
         language=None,
         no_create=False,
-        **fields
+        **fields,
     ):
         """
         Internal add_document used for both batch and single doc indexing
         """
-        if conn is None:
-            conn = self.client
 
         if partial or no_create:
             replace = True
@@ -179,21 +211,18 @@ class SearchCommands:
             args += ["LANGUAGE", language]
         args.append("FIELDS")
         args += list(itertools.chain(*fields.items()))
-        return conn.execute_command(*args)
+
+        if conn is not None:
+            return conn.execute_command(*args)
+
+        return self.execute_command(*args)
 
     def _add_document_hash(
-        self,
-        doc_id,
-        conn=None,
-        score=1.0,
-        language=None,
-        replace=False,
+        self, doc_id, conn=None, score=1.0, language=None, replace=False
     ):
         """
         Internal add_document_hash used for both batch and single doc indexing
         """
-        if conn is None:
-            conn = self.client
 
         args = [ADDHASH_CMD, self.index_name, doc_id, score]
 
@@ -203,8 +232,14 @@ class SearchCommands:
         if language:
             args += ["LANGUAGE", language]
 
-        return conn.execute_command(*args)
+        if conn is not None:
+            return conn.execute_command(*args)
 
+        return self.execute_command(*args)
+
+    @deprecated_function(
+        version="2.0.0", reason="deprecated since redisearch 2.0, call hset instead"
+    )
     def add_document(
         self,
         doc_id,
@@ -215,7 +250,7 @@ class SearchCommands:
         partial=False,
         language=None,
         no_create=False,
-        **fields
+        **fields,
     ):
         """
         Add a single document to the index.
@@ -244,7 +279,7 @@ class SearchCommands:
         - **fields** kwargs dictionary of the document fields to be saved
                          and/or indexed.
                      NOTE: Geo points shoule be encoded as strings of "lon,lat"
-        """
+        """  # noqa
         return self._add_document(
             doc_id,
             conn=None,
@@ -255,16 +290,13 @@ class SearchCommands:
             partial=partial,
             language=language,
             no_create=no_create,
-            **fields
+            **fields,
         )
 
-    def add_document_hash(
-        self,
-        doc_id,
-        score=1.0,
-        language=None,
-        replace=False,
-    ):
+    @deprecated_function(
+        version="2.0.0", reason="deprecated since redisearch 2.0, call hset instead"
+    )
+    def add_document_hash(self, doc_id, score=1.0, language=None, replace=False):
         """
         Add a hash document to the index.
 
@@ -276,13 +308,9 @@ class SearchCommands:
         - **replace**: if True, and the document already is in the index, we
                       perform an update and reindex the document
         - **language**: Specify the language used for document tokenization.
-        """
+        """  # noqa
         return self._add_document_hash(
-            doc_id,
-            conn=None,
-            score=score,
-            language=language,
-            replace=replace,
+            doc_id, conn=None, score=score, language=language, replace=replace
         )
 
     def delete_document(self, doc_id, conn=None, delete_actual_document=False):
@@ -294,14 +322,15 @@ class SearchCommands:
 
         - **delete_actual_document**: if set to True, RediSearch also delete
                                       the actual document if it is in the index
-        """
+        """  # noqa
         args = [DEL_CMD, self.index_name, doc_id]
-        if conn is None:
-            conn = self.client
         if delete_actual_document:
             args.append("DD")
 
-        return conn.execute_command(*args)
+        if conn is not None:
+            return conn.execute_command(*args)
+
+        return self.execute_command(*args)
 
     def load_document(self, id):
         """
@@ -325,33 +354,56 @@ class SearchCommands:
         ### Parameters
 
         - **ids**: the ids of the saved documents.
+
         """
 
-        return self.client.execute_command(MGET_CMD, self.index_name, *ids)
+        return self.execute_command(MGET_CMD, self.index_name, *ids)
 
     def info(self):
         """
         Get info an stats about the the current index, including the number of
         documents, memory consumption, etc
+
+        For more information see `FT.INFO <https://redis.io/commands/ft.info>`_.
         """
 
-        res = self.client.execute_command(INFO_CMD, self.index_name)
+        res = self.execute_command(INFO_CMD, self.index_name)
         it = map(to_string, res)
         return dict(zip(it, it))
 
-    def _mk_query_args(self, query):
+    def get_params_args(
+        self, query_params: Union[Dict[str, Union[str, int, float]], None]
+    ):
+        if query_params is None:
+            return []
+        args = []
+        if len(query_params) > 0:
+            args.append("params")
+            args.append(len(query_params) * 2)
+            for key, value in query_params.items():
+                args.append(key)
+                args.append(value)
+        return args
+
+    def _mk_query_args(self, query, query_params: Dict[str, Union[str, int, float]]):
         args = [self.index_name]
 
         if isinstance(query, str):
             # convert the query from a text to a query object
             query = Query(query)
         if not isinstance(query, Query):
-            raise ValueError("Bad query type %s" % type(query))
+            raise ValueError(f"Bad query type {type(query)}")
 
         args += query.get_args()
+        args += self.get_params_args(query_params)
+
         return args, query
 
-    def search(self, query):
+    def search(
+        self,
+        query: Union[str, Query],
+        query_params: Dict[str, Union[str, int, float]] = None,
+    ):
         """
         Search the index for a given query, and return a result of documents
 
@@ -360,11 +412,16 @@ class SearchCommands:
         - **query**: the search query. Either a text for simple queries with
                      default parameters, or a Query object for complex queries.
                      See RediSearch's documentation on query format
-        """
-        args, query = self._mk_query_args(query)
+
+        For more information see `FT.SEARCH <https://redis.io/commands/ft.search>`_.
+        """  # noqa
+        args, query = self._mk_query_args(query, query_params=query_params)
         st = time.time()
         res = self.execute_command(SEARCH_CMD, *args)
 
+        if isinstance(res, Pipeline):
+            return res
+
         return Result(
             res,
             not query._no_content,
@@ -373,24 +430,38 @@ class SearchCommands:
             with_scores=query._with_scores,
         )
 
-    def explain(self, query):
-        args, query_text = self._mk_query_args(query)
+    def explain(
+        self,
+        query: Union[str, Query],
+        query_params: Dict[str, Union[str, int, float]] = None,
+    ):
+        """Returns the execution plan for a complex query.
+
+        For more information see `FT.EXPLAIN <https://redis.io/commands/ft.explain>`_.
+        """  # noqa
+        args, query_text = self._mk_query_args(query, query_params=query_params)
         return self.execute_command(EXPLAIN_CMD, *args)
 
-    def explain_cli(self, query):  # noqa
+    def explain_cli(self, query: Union[str, Query]):  # noqa
         raise NotImplementedError("EXPLAINCLI will not be implemented.")
 
-    def aggregate(self, query):
+    def aggregate(
+        self,
+        query: Union[str, Query],
+        query_params: Dict[str, Union[str, int, float]] = None,
+    ):
         """
-        Issue an aggregation query
+        Issue an aggregation query.
 
         ### Parameters
 
-        **query**: This can be either an `AggeregateRequest`, or a `Cursor`
+        **query**: This can be either an `AggregateRequest`, or a `Cursor`
 
         An `AggregateResult` object is returned. You can access the rows from
         its `rows` property, which will always yield the rows of the result.
-        """
+
+        For more information see `FT.AGGREGATE <https://redis.io/commands/ft.aggregate>`_.
+        """  # noqa
         if isinstance(query, AggregateRequest):
             has_cursor = bool(query._cursor)
             cmd = [AGGREGATE_CMD, self.index_name] + query.build_args()
@@ -399,8 +470,12 @@ class SearchCommands:
             cmd = [CURSOR_CMD, "READ", self.index_name] + query.build_args()
         else:
             raise ValueError("Bad query", query)
+        cmd += self.get_params_args(query_params)
 
         raw = self.execute_command(*cmd)
+        return self._get_aggregate_result(raw, query, has_cursor)
+
+    def _get_aggregate_result(self, raw, query, has_cursor):
         if has_cursor:
             if isinstance(query, Cursor):
                 query.cid = raw[1]
@@ -418,8 +493,56 @@ class SearchCommands:
             schema = None
             rows = raw[1:]
 
-        res = AggregateResult(rows, cursor, schema)
-        return res
+        return AggregateResult(rows, cursor, schema)
+
+    def profile(
+        self,
+        query: Union[str, Query, AggregateRequest],
+        limited: bool = False,
+        query_params: Optional[Dict[str, Union[str, int, float]]] = None,
+    ):
+        """
+        Performs a search or aggregate command and collects performance
+        information.
+
+        ### Parameters
+
+        **query**: This can be either an `AggregateRequest`, `Query` or string.
+        **limited**: If set to True, removes details of reader iterator.
+        **query_params**: Define one or more value parameters.
+        Each parameter has a name and a value.
+
+        """
+        st = time.time()
+        cmd = [PROFILE_CMD, self.index_name, ""]
+        if limited:
+            cmd.append("LIMITED")
+        cmd.append("QUERY")
+
+        if isinstance(query, AggregateRequest):
+            cmd[2] = "AGGREGATE"
+            cmd += query.build_args()
+        elif isinstance(query, Query):
+            cmd[2] = "SEARCH"
+            cmd += query.get_args()
+            cmd += self.get_params_args(query_params)
+        else:
+            raise ValueError("Must provide AggregateRequest object or " "Query object.")
+
+        res = self.execute_command(*cmd)
+
+        if isinstance(query, AggregateRequest):
+            result = self._get_aggregate_result(res[0], query, query._cursor)
+        else:
+            result = Result(
+                res[0],
+                not query._no_content,
+                duration=(time.time() - st) * 1000.0,
+                has_payload=query._with_payloads,
+                with_scores=query._with_scores,
+            )
+
+        return result, parse_to_dict(res[1])
 
     def spellcheck(self, query, distance=None, include=None, exclude=None):
         """
@@ -432,7 +555,9 @@ class SearchCommands:
                        suggestions (default: 1, max: 4).
         **include**: specifies an inclusion custom dictionary.
         **exclude**: specifies an exclusion custom dictionary.
-        """
+
+        For more information see `FT.SPELLCHECK <https://redis.io/commands/ft.spellcheck>`_.
+        """  # noqa
         cmd = [SPELLCHECK_CMD, self.index_name, query]
         if distance:
             cmd.extend(["DISTANCE", distance])
@@ -476,8 +601,7 @@ class SearchCommands:
             #     ]
             # }
             corrections[_correction[1]] = [
-                {"score": _item[0], "suggestion": _item[1]}
-                for _item in _correction[2]
+                {"score": _item[0], "suggestion": _item[1]} for _item in _correction[2]
             ]
 
         return corrections
@@ -489,7 +613,9 @@ class SearchCommands:
 
         - **name**: Dictionary name.
         - **terms**: List of items for adding to the dictionary.
-        """
+
+        For more information see `FT.DICTADD <https://redis.io/commands/ft.dictadd>`_.
+        """  # noqa
         cmd = [DICT_ADD_CMD, name]
         cmd.extend(terms)
         return self.execute_command(*cmd)
@@ -501,7 +627,9 @@ class SearchCommands:
 
         - **name**: Dictionary name.
         - **terms**: List of items for removing from the dictionary.
-        """
+
+        For more information see `FT.DICTDEL <https://redis.io/commands/ft.dictdel>`_.
+        """  # noqa
         cmd = [DICT_DEL_CMD, name]
         cmd.extend(terms)
         return self.execute_command(*cmd)
@@ -512,7 +640,9 @@ class SearchCommands:
         ### Parameters
 
         - **name**: Dictionary name.
-        """
+
+        For more information see `FT.DICTDUMP <https://redis.io/commands/ft.dictdump>`_.
+        """  # noqa
         cmd = [DICT_DUMP_CMD, name]
         return self.execute_command(*cmd)
 
@@ -523,7 +653,9 @@ class SearchCommands:
 
         - **option**: the name of the configuration option.
         - **value**: a value for the configuration option.
-        """
+
+        For more information see `FT.CONFIG SET <https://redis.io/commands/ft.config-set>`_.
+        """  # noqa
         cmd = [CONFIG_CMD, "SET", option, value]
         raw = self.execute_command(*cmd)
         return raw == "OK"
@@ -534,7 +666,9 @@ class SearchCommands:
         ### Parameters
 
         - **option**: the name of the configuration option.
-        """
+
+        For more information see `FT.CONFIG GET <https://redis.io/commands/ft.config-get>`_.
+        """  # noqa
         cmd = [CONFIG_CMD, "GET", option]
         res = {}
         raw = self.execute_command(*cmd)
@@ -550,7 +684,9 @@ class SearchCommands:
         ### Parameters
 
         - **tagfield**: Tag field name
-        """
+
+        For more information see `FT.TAGVALS <https://redis.io/commands/ft.tagvals>`_.
+        """  # noqa
 
         return self.execute_command(TAGVALS_CMD, self.index_name, tagfield)
 
@@ -561,7 +697,9 @@ class SearchCommands:
         ### Parameters
 
         - **alias**: Name of the alias to create
-        """
+
+        For more information see `FT.ALIASADD <https://redis.io/commands/ft.aliasadd>`_.
+        """  # noqa
 
         return self.execute_command(ALIAS_ADD_CMD, alias, self.index_name)
 
@@ -572,7 +710,9 @@ class SearchCommands:
         ### Parameters
 
         - **alias**: Name of the alias to create
-        """
+
+        For more information see `FT.ALIASUPDATE <https://redis.io/commands/ft.aliasupdate>`_.
+        """  # noqa
 
         return self.execute_command(ALIAS_UPDATE_CMD, alias, self.index_name)
 
@@ -583,7 +723,9 @@ class SearchCommands:
         ### Parameters
 
         - **alias**: Name of the alias to delete
-        """
+
+        For more information see `FT.ALIASDEL <https://redis.io/commands/ft.aliasdel>`_.
+        """  # noqa
         return self.execute_command(ALIAS_DEL_CMD, alias)
 
     def sugadd(self, key, *suggestions, **kwargs):
@@ -592,8 +734,9 @@ class SearchCommands:
         a score and string.
         If kwargs["increment"] is true and the terms are already in the
         server's dictionary, we increment their scores.
-        More information `here <https://oss.redis.com/redisearch/master/Commands/#ftsugadd>`_.  # noqa
-        """
+
+        For more information see `FT.SUGADD <https://redis.io/commands/ft.sugadd/>`_.
+        """  # noqa
         # If Transaction is not False it will MULTI/EXEC which will error
         pipe = self.pipeline(transaction=False)
         for sug in suggestions:
@@ -611,25 +754,25 @@ class SearchCommands:
     def suglen(self, key):
         """
         Return the number of entries in the AutoCompleter index.
-        More information `here <https://oss.redis.com/redisearch/master/Commands/#ftsuglen>`_.  # noqa
-        """
+
+        For more information see `FT.SUGLEN <https://redis.io/commands/ft.suglen>`_.
+        """  # noqa
         return self.execute_command(SUGLEN_COMMAND, key)
 
     def sugdel(self, key, string):
         """
         Delete a string from the AutoCompleter index.
         Returns 1 if the string was found and deleted, 0 otherwise.
-        More information `here <https://oss.redis.com/redisearch/master/Commands/#ftsugdel>`_.  # noqa
-        """
+
+        For more information see `FT.SUGDEL <https://redis.io/commands/ft.sugdel>`_.
+        """  # noqa
         return self.execute_command(SUGDEL_COMMAND, key, string)
 
     def sugget(
-        self, key, prefix, fuzzy=False, num=10, with_scores=False,
-        with_payloads=False
+        self, key, prefix, fuzzy=False, num=10, with_scores=False, with_payloads=False
     ):
         """
         Get a list of suggestions from the AutoCompleter, for a given prefix.
-        More information `here <https://oss.redis.com/redisearch/master/Commands/#ftsugget>`_.  # noqa
 
         Parameters:
 
@@ -656,7 +799,9 @@ class SearchCommands:
         list:
              A list of Suggestion objects. If with_scores was False, the
              score of all suggestions is 1.
-        """
+
+        For more information see `FT.SUGGET <https://redis.io/commands/ft.sugget>`_.
+        """  # noqa
         args = [SUGGET_COMMAND, key, prefix, "MAX", num]
         if fuzzy:
             args.append(FUZZY)
@@ -688,7 +833,9 @@ class SearchCommands:
             If set to true, we do not scan and index.
         terms :
             The terms.
-        """
+
+        For more information see `FT.SYNUPDATE <https://redis.io/commands/ft.synupdate>`_.
+        """  # noqa
         cmd = [SYNUPDATE_CMD, self.index_name, groupid]
         if skipinitial:
             cmd.extend(["SKIPINITIALSCAN"])
@@ -701,6 +848,249 @@ class SearchCommands:
 
         The command is used to dump the synonyms data structure.
         Returns a list of synonym terms and their synonym group ids.
-        """
+
+        For more information see `FT.SYNDUMP <https://redis.io/commands/ft.syndump>`_.
+        """  # noqa
         raw = self.execute_command(SYNDUMP_CMD, self.index_name)
         return {raw[i]: raw[i + 1] for i in range(0, len(raw), 2)}
+
+
+class AsyncSearchCommands(SearchCommands):
+    async def info(self):
+        """
+        Get info an stats about the the current index, including the number of
+        documents, memory consumption, etc
+
+        For more information see `FT.INFO <https://redis.io/commands/ft.info>`_.
+        """
+
+        res = await self.execute_command(INFO_CMD, self.index_name)
+        it = map(to_string, res)
+        return dict(zip(it, it))
+
+    async def search(
+        self,
+        query: Union[str, Query],
+        query_params: Dict[str, Union[str, int, float]] = None,
+    ):
+        """
+        Search the index for a given query, and return a result of documents
+
+        ### Parameters
+
+        - **query**: the search query. Either a text for simple queries with
+                     default parameters, or a Query object for complex queries.
+                     See RediSearch's documentation on query format
+
+        For more information see `FT.SEARCH <https://redis.io/commands/ft.search>`_.
+        """  # noqa
+        args, query = self._mk_query_args(query, query_params=query_params)
+        st = time.time()
+        res = await self.execute_command(SEARCH_CMD, *args)
+
+        if isinstance(res, Pipeline):
+            return res
+
+        return Result(
+            res,
+            not query._no_content,
+            duration=(time.time() - st) * 1000.0,
+            has_payload=query._with_payloads,
+            with_scores=query._with_scores,
+        )
+
+    async def aggregate(
+        self,
+        query: Union[str, Query],
+        query_params: Dict[str, Union[str, int, float]] = None,
+    ):
+        """
+        Issue an aggregation query.
+
+        ### Parameters
+
+        **query**: This can be either an `AggregateRequest`, or a `Cursor`
+
+        An `AggregateResult` object is returned. You can access the rows from
+        its `rows` property, which will always yield the rows of the result.
+
+        For more information see `FT.AGGREGATE <https://redis.io/commands/ft.aggregate>`_.
+        """  # noqa
+        if isinstance(query, AggregateRequest):
+            has_cursor = bool(query._cursor)
+            cmd = [AGGREGATE_CMD, self.index_name] + query.build_args()
+        elif isinstance(query, Cursor):
+            has_cursor = True
+            cmd = [CURSOR_CMD, "READ", self.index_name] + query.build_args()
+        else:
+            raise ValueError("Bad query", query)
+        cmd += self.get_params_args(query_params)
+
+        raw = await self.execute_command(*cmd)
+        return self._get_aggregate_result(raw, query, has_cursor)
+
+    async def spellcheck(self, query, distance=None, include=None, exclude=None):
+        """
+        Issue a spellcheck query
+
+        ### Parameters
+
+        **query**: search query.
+        **distance***: the maximal Levenshtein distance for spelling
+                       suggestions (default: 1, max: 4).
+        **include**: specifies an inclusion custom dictionary.
+        **exclude**: specifies an exclusion custom dictionary.
+
+        For more information see `FT.SPELLCHECK <https://redis.io/commands/ft.spellcheck>`_.
+        """  # noqa
+        cmd = [SPELLCHECK_CMD, self.index_name, query]
+        if distance:
+            cmd.extend(["DISTANCE", distance])
+
+        if include:
+            cmd.extend(["TERMS", "INCLUDE", include])
+
+        if exclude:
+            cmd.extend(["TERMS", "EXCLUDE", exclude])
+
+        raw = await self.execute_command(*cmd)
+
+        corrections = {}
+        if raw == 0:
+            return corrections
+
+        for _correction in raw:
+            if isinstance(_correction, int) and _correction == 0:
+                continue
+
+            if len(_correction) != 3:
+                continue
+            if not _correction[2]:
+                continue
+            if not _correction[2][0]:
+                continue
+
+            corrections[_correction[1]] = [
+                {"score": _item[0], "suggestion": _item[1]} for _item in _correction[2]
+            ]
+
+        return corrections
+
+    async def config_set(self, option, value):
+        """Set runtime configuration option.
+
+        ### Parameters
+
+        - **option**: the name of the configuration option.
+        - **value**: a value for the configuration option.
+
+        For more information see `FT.CONFIG SET <https://redis.io/commands/ft.config-set>`_.
+        """  # noqa
+        cmd = [CONFIG_CMD, "SET", option, value]
+        raw = await self.execute_command(*cmd)
+        return raw == "OK"
+
+    async def config_get(self, option):
+        """Get runtime configuration option value.
+
+        ### Parameters
+
+        - **option**: the name of the configuration option.
+
+        For more information see `FT.CONFIG GET <https://redis.io/commands/ft.config-get>`_.
+        """  # noqa
+        cmd = [CONFIG_CMD, "GET", option]
+        res = {}
+        raw = await self.execute_command(*cmd)
+        if raw:
+            for kvs in raw:
+                res[kvs[0]] = kvs[1]
+        return res
+
+    async def load_document(self, id):
+        """
+        Load a single document by id
+        """
+        fields = await self.client.hgetall(id)
+        f2 = {to_string(k): to_string(v) for k, v in fields.items()}
+        fields = f2
+
+        try:
+            del fields["id"]
+        except KeyError:
+            pass
+
+        return Document(id=id, **fields)
+
+    async def sugadd(self, key, *suggestions, **kwargs):
+        """
+        Add suggestion terms to the AutoCompleter engine. Each suggestion has
+        a score and string.
+        If kwargs["increment"] is true and the terms are already in the
+        server's dictionary, we increment their scores.
+
+        For more information see `FT.SUGADD <https://redis.io/commands/ft.sugadd>`_.
+        """  # noqa
+        # If Transaction is not False it will MULTI/EXEC which will error
+        pipe = self.pipeline(transaction=False)
+        for sug in suggestions:
+            args = [SUGADD_COMMAND, key, sug.string, sug.score]
+            if kwargs.get("increment"):
+                args.append("INCR")
+            if sug.payload:
+                args.append("PAYLOAD")
+                args.append(sug.payload)
+
+            pipe.execute_command(*args)
+
+        return (await pipe.execute())[-1]
+
+    async def sugget(
+        self, key, prefix, fuzzy=False, num=10, with_scores=False, with_payloads=False
+    ):
+        """
+        Get a list of suggestions from the AutoCompleter, for a given prefix.
+
+        Parameters:
+
+        prefix : str
+            The prefix we are searching. **Must be valid ascii or utf-8**
+        fuzzy : bool
+            If set to true, the prefix search is done in fuzzy mode.
+            **NOTE**: Running fuzzy searches on short (<3 letters) prefixes
+            can be very
+            slow, and even scan the entire index.
+        with_scores : bool
+            If set to true, we also return the (refactored) score of
+            each suggestion.
+            This is normally not needed, and is NOT the original score
+            inserted into the index.
+        with_payloads : bool
+            Return suggestion payloads
+        num : int
+            The maximum number of results we return. Note that we might
+            return less. The algorithm trims irrelevant suggestions.
+
+        Returns:
+
+        list:
+             A list of Suggestion objects. If with_scores was False, the
+             score of all suggestions is 1.
+
+        For more information see `FT.SUGGET <https://redis.io/commands/ft.sugget>`_.
+        """  # noqa
+        args = [SUGGET_COMMAND, key, prefix, "MAX", num]
+        if fuzzy:
+            args.append(FUZZY)
+        if with_scores:
+            args.append(WITHSCORES)
+        if with_payloads:
+            args.append(WITHPAYLOADS)
+
+        ret = await self.execute_command(*args)
+        results = []
+        if not ret:
+            return results
+
+        parser = SuggestionParser(with_scores, with_payloads, ret)
+        return [s for s in parser]
